import 'package:cramr_ui_lib/ads/preloaded_banner_ad.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:google_mobile_ads/google_mobile_ads.dart';
import 'package:provider/provider.dart';

import 'ads_controller.dart';
import 'loading_state.dart';

abstract class AbstractBannerAdWidget extends StatefulWidget {
  const AbstractBannerAdWidget({Key? key, required this.adUnitId})
      : super(key: key);

  final String adUnitId;

  @override
  State<AbstractBannerAdWidget> createState() => AbstractBannerAdWidgetState();
}

class AbstractBannerAdWidgetState extends State<AbstractBannerAdWidget> {
  BannerAd? _bannerAd;
  LoadingState _adLoadingState = LoadingState.initial;

  @override
  Widget build(BuildContext context) {
    if (_bannerAd != null && _adLoadingState == LoadingState.loaded) {
      return SizedBox(
        width: _bannerAd!.size.width.toDouble(),
        height: _bannerAd!.size.height.toDouble(),
        child: AdWidget(ad: _bannerAd!),
      );
    }
    return const SizedBox();
  }

  @override
  void initState() {
    super.initState();

    final adsController = context.read<AdsController>();
    final ad = adsController.takePreloadedAd(widget.adUnitId);
    if (ad != null) {
      _showPreloadedAd(ad);
    } else {
      _loadAd();
    }
  }

  void _showPreloadedAd(PreloadedBannerAd ad) async {
    // It's possible that the banner is still loading (even though it started
    // preloading at the start of the previous screen).
    _adLoadingState = LoadingState.loading;
    try {
      _bannerAd = await ad.ready;
    } on LoadAdError catch (error) {
      if (kDebugMode) {
        print('error is ****************** $error.message');
      }
      _loadAd();
      return;
    }
    if (!mounted) return;
    if (!ad.isLoaded) return;

    setState(() {
      _adLoadingState = LoadingState.loaded;
    });
  }

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
  }

  @override
  void dispose() {
    _bannerAd?.dispose();
    super.dispose();
  }

  /// Load (another) ad, disposing of the current ad if there is one.
  Future<void> _loadAd() async {
    if (_adLoadingState == LoadingState.loading ||
        _adLoadingState == LoadingState.disposing) {
      return;
    }
    _adLoadingState = LoadingState.disposing;
    await _bannerAd?.dispose();
    if (!mounted) return;
    setState(() {
      _bannerAd = null;
      _adLoadingState = LoadingState.loading;
    });

    AdSize size;

    size = AdSize.banner;

    if (!mounted) return;

    // assert(Platform.isAndroid || Platform.isIOS,
    //     'AdMob currently does not support ${Platform.operatingSystem}');
    final AdsController? adsController = context.read<AdsController?>();
    _bannerAd = BannerAd(
      // This is a test ad unit ID from
      // https://developers.google.com/admob/android/test-ads. When ready,
      // you replace this with your own, production ad unit ID,
      // created in https://apps.admob.com/.
      adUnitId: widget.adUnitId,
      size: size,
      request: adsController != null
          ? adsController.createAdRequest()
          : const AdRequest(),
      listener: BannerAdListener(
        onAdLoaded: (Ad ad) {
          setState(() {
            // When the ad is loaded, get the ad size and use it to set
            // the height of the ad container.
            _bannerAd = ad as BannerAd;
            _adLoadingState = LoadingState.loaded;
          });
        },
        onAdFailedToLoad: (Ad ad, LoadAdError error) {
          ad.dispose();
        },
        onAdImpression: (Ad ad) {},
        onAdClicked: (Ad ad) {},
      ),
    );
    return _bannerAd!.load();
  }
}
