import 'dart:collection';

import 'package:cramr_ui_lib/ads/safe_banner_ad_widget.dart';
import 'package:cramr_ui_lib/model/cardinality_selection.dart';
import 'package:cramr_ui_lib/model/verb.dart';
import 'package:cramr_ui_lib/model/verb_test_model.dart';
import 'package:cramr_ui_lib/repo/verb_repository.dart';
import 'package:cramr_ui_lib/routes/verb_studymode_selection_route.dart';
import 'package:cramr_ui_lib/settings/settings_controller.dart';
import 'package:cramr_ui_lib/settings/settings_view.dart';
import 'package:cramr_ui_lib/util/cardinality_filter.dart';
import 'package:cramr_ui_lib/util/text_size_limiter.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import '../widgets/verb_card.dart';

/// The Route that allows you to select a verb for evaluation
class ConjugationSelectionRoute extends StatefulWidget {
  static const routeName = '/conjugationSelectionRoute';

  final SettingsController _settingsController;
  final SafeBannerAdWidget safeBannerAdWidget;

  const ConjugationSelectionRoute(this._settingsController,
      {Key? key, required this.safeBannerAdWidget})
      : super(key: key);

  @override
  State<ConjugationSelectionRoute> createState() =>
      ConjugationSelectionRouteState();

  static Key verbIconKey(String infinitive, String mode, String tense) {
    return Key('startStudy_${infinitive}_${mode}_$tense');
  }
}

class ConjugationSelectionRouteState extends State<ConjugationSelectionRoute> {
  final HashMap<String, Verb> verbs = VerbRepository.instance().verbs;

  List<String>? filteredVerbs = [];
  bool isByVerbList = true;

  late List<VerbPanelItem> _verbPanelItems;

  HashMap<String, List<String>> _tensesPerMode() {
    //two iterations over the verbs:
    //first to make sure we have a set of all Conjugations available
    //secondly we append the verbs to the conjugations
    HashMap<String, List<String>> conjugations = HashMap();
    for (var verb in verbs.values) {
      for (var conjugation in verb.indicativeConjugations.values) {
        if (!conjugations.containsKey(conjugation.mode)) {
          conjugations[conjugation.mode] = [conjugation.tense];
        } else {
          // mode is present, check for conjugation
          List<String>? modeConjugations = conjugations[conjugation.mode];
          if (null != modeConjugations) {
            if (!modeConjugations.contains(conjugation.tense)) {
              modeConjugations.add(conjugation.tense);
            }
          }
        }
      }
    }
    return conjugations;
  }

  List<VerbPanelItem> generateItems() {
    HashMap<String, List<String>> conjugations = _tensesPerMode();
    List<VerbPanelItem> panelItems = [];
    for (String mode in conjugations.keys) {
      var conjugation = conjugations[mode];
      if (conjugation != null) {
        for (String tense in conjugation) {
          panelItems.add(VerbPanelItem(mode: mode, tense: tense));
        }
      }
    }
    return panelItems;
  }

  @override
  Widget build(BuildContext context) {
    return AnimatedBuilder(
        animation: widget._settingsController,
        builder: (BuildContext context, Widget? child) {
          return MaterialApp(
            debugShowCheckedModeBanner: false,
            themeMode: widget._settingsController.themeMode,
            theme: ThemeData(),
            darkTheme: ThemeData.dark(),
            home: SafeArea(
              child: Scaffold(
                appBar: AppBar(
                  leading: IconButton(
                      key: const Key('BackKey'),
                      onPressed: () {
                        Navigator.pop(context);
                      },
                      icon: const Icon(Icons.arrow_back)),
                  actions: [_settingsIcon(context)],
                ),
                body: Column(
                  children: [
                    Expanded(
                      child: Center(
                        key: const Key('verbSelectionCenter'),
                        child: _verbListView(context),
                      ),
                    ),
                    Container(
                      height: 100.0,
                      alignment: Alignment.center,
                      child: widget.safeBannerAdWidget,
                    )
                  ],
                ),
                // This trailing comma makes auto-formatting nicer for build methods.
              ),
            ),
          );
        });
  }

  Widget _verbListView(BuildContext context) {
    return SingleChildScrollView(
      padding: const EdgeInsets.all(25),
      child: _conjugationList(context),
    );
  }

  Widget _conjugationList(BuildContext context) {
    return ExpansionPanelList(
      key: const Key('conjugationListView'),
      expandedHeaderPadding: const EdgeInsets.all(25),
      expansionCallback: (int index, bool isExpanded) {
        setState(() {
          _verbPanelItems[index].isExpanded = !isExpanded;
        });
      },
      children: _verbPanelItems.map<ExpansionPanel>((VerbPanelItem item) {
        return ExpansionPanel(
            canTapOnHeader: true,
            headerBuilder: (BuildContext context, bool isExpanded) {
              String mode = item.mode;
              String tense = item.tense;
              return ListTile(
                key: Key('vervoegTile_${mode}_$tense'),
                contentPadding: const EdgeInsets.fromLTRB(20, 15, 40, 15),
                title: SizedBox(
                  key: UniqueKey(),
                  height: 80,
                  child: Text(
                    '$mode $tense',
                    key: Key('vervoeg_${mode}_$tense'),
                    style: Theme.of(context).textTheme.headline5,
                    textScaleFactor: TextSizeLimiter.textScaleFactor(context),
                  ),
                ),
              );
            },
            body: ListTile(
              title: _conjugationButton(context, item.mode, item.tense),
            ),
            isExpanded: item.isExpanded);
      }).toList(),
    );
  }

  IconButton _settingsIcon(BuildContext context) {
    return IconButton(
        key: const Key('settingsButton'),
        icon: const Icon(Icons.settings),
        onPressed: () {
          Navigator.pushNamed(context, SettingsView.routeName);
        });
  }

  Widget _conjugationButton(BuildContext context, String mode, String tense) {
    return Padding(
        padding: const EdgeInsets.all(8.0),
        child: Column(
          children: [
            Card(
              elevation: 5.0,
              child: Column(children: _verbCards(context, mode, tense)),
            )
          ],
        ));
  }

  List<Widget> _verbCards(BuildContext context, String mode, String tense) {
    List<Widget> result = [];
    for (var key in filteredVerbs!) {
      if (null != verbs[key]) {
        Verb? verb = verbs[key];
        result.add(_verbCard(context, mode, tense, verb!));
      }
    }
    return result;
  }

  Widget _verbCard(BuildContext context, String mode, String tense, Verb verb) {
    return VerbCard(
        infinitive: verb.infinitive,
        verbIcon: _verbIcon(context, verb, mode, tense));
  }

  Widget _verbIcon(
      BuildContext context, Verb verb, String? mode, String? tense) {
    Key iconKey = Key('startStudy_${verb.infinitive}');
    List<String>? indicative = verb.simplePresentIndicative;
    if (null != tense) {
      iconKey =
          ConjugationSelectionRoute.verbIconKey(verb.infinitive, mode!, tense);
      indicative = verb.indicativeConjugations['${mode}_$tense']!.conjugation;
    }
    return InkWell(
      key: UniqueKey(),
      child: CircleAvatar(
          key: UniqueKey(), child: Icon(Icons.play_arrow, key: iconKey)),
      onTap: () {
        final VerbTestModel model =
            Provider.of<VerbTestModel>(context, listen: false);
        final CardinalitySelection selection =
            widget._settingsController.cardinalitySelection;
        model.initialise(
            CardinalityFilter.filteredList(selection, indicative!),
            CardinalityFilter.filteredList(
                selection, VerbRepository.instance().pronouns),
            verb.infinitive,
            verb.isoLanguage,
            verb.translation,
            mode!,
            tense!);
        model.reset();
        Navigator.pushNamed(context, VerbStudymodeSelectionRoute.routeName);
      },
    );
  }

  @override
  void initState() {
    super.initState();
    verbs.forEach((key, value) {
      filteredVerbs!.add(key);
    });

    _verbPanelItems = generateItems();
  }
}

class VerbSearchDelegate extends SearchDelegate<String> {
  List<String> allVerbs = [];
  List<String> suggestedVerbs = [];

  VerbSearchDelegate() {
    var verbs = VerbRepository.instance().verbs;
    for (var element in verbs.keys) {
      allVerbs.add(element);
    }
  }

  @override
  List<Widget>? buildActions(BuildContext context) {
    List<Widget> actions = [];
    return actions;
  }

  @override
  Widget? buildLeading(BuildContext context) {
    return null;
  }

  @override
  Widget buildResults(BuildContext context) {
    for (var element in allVerbs) {
      if (element.toLowerCase().contains(query.toLowerCase())) {
        suggestedVerbs.add(element);
      }
    }

    return ListView.builder(
        itemCount: suggestedVerbs.length,
        itemBuilder: (BuildContext context, int index) {
          var result = suggestedVerbs[index];
          return ListTile(
            title: Text(
              result,
              key: Key("searchResult_$result"),
              textScaleFactor: TextSizeLimiter.textScaleFactor(context),
            ),
            onTap: () {
              close(context, result);
            },
          );
        });
  }

  @override
  Widget buildSuggestions(BuildContext context) {
    suggestedVerbs.clear();
    for (var element in allVerbs) {
      if (element.toLowerCase().contains(query.toLowerCase())) {
        suggestedVerbs.add(element);
      }
    }

    return ListView.builder(
        itemCount: suggestedVerbs.length,
        itemBuilder: (BuildContext context, int index) {
          var result = suggestedVerbs[index];
          return ListTile(
            title: Text(
              result,
              key: Key("suggestedResult_$result"),
              textScaleFactor: TextSizeLimiter.textScaleFactor(context),
            ),
            onTap: () {
              close(context, result);
            },
          );
        });
  }
}

class VerbPanelItem {
  late final String mode;
  late final String tense;
  bool isExpanded;

  VerbPanelItem(
      {required this.mode, required this.tense, this.isExpanded = false});
}
