import 'package:cramr_ui_lib/model/verb_test_model.dart';
import 'package:cramr_ui_lib/routes/verb_selection_route.dart';
import 'package:cramr_ui_lib/routes/verb_studymode_selection_route.dart';
import 'package:cramr_ui_lib/settings/settings_controller.dart';
import 'package:cramr_ui_lib/settings/settings_view.dart';
import 'package:cramr_ui_lib/widgets/score_widget.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

import '../widgets/conjugation_meta_data_card.dart';
import '../widgets/evalutation_row.dart';

/// The Route that allows you to show the evaluation of the indicative of the
/// [Verb] that you are studying.
class VerbIndicativeEvaluateRoute extends StatefulWidget {
  static const routeName = '/verbEvaluateRoute';

  final SettingsController _settingsController;

  const VerbIndicativeEvaluateRoute(this._settingsController, {Key? key})
      : super(key: key);

  @override
  State<VerbIndicativeEvaluateRoute> createState() =>
      VerbIndicativeEvaluateRouteState();
}

class VerbIndicativeEvaluateRouteState
    extends State<VerbIndicativeEvaluateRoute> {
  @override
  Widget build(BuildContext context) {
    return AnimatedBuilder(
      animation: widget._settingsController,
      builder: (BuildContext context, Widget? child) {
        return MaterialApp(
          debugShowCheckedModeBanner: false,
          themeMode: widget._settingsController.themeMode,
          theme: ThemeData(),
          darkTheme: ThemeData.dark(),
          home: SafeArea(
            child: Scaffold(
              appBar: AppBar(
                actions: [
                  _repeatIcon(context),
                  _homeIcon(context),
                  _settingsIcon(context)
                ],
                title: _titleCell(context),
              ),
              body: _centerWidget(context),
            ),
          ),
        );
      },
    );
  }

  Widget _centerWidget(BuildContext context) {
    return SafeArea(
      child: Scaffold(
        key: UniqueKey(),
        body: SingleChildScrollView(child: _bodyListView(context)),
      ),
    );
  }

  Widget _bodyListView(BuildContext context) {
    widget._settingsController.updateStreakEngine();
    return Column(
      children: [
        _conjugationMetaDataCard(context),
        const Padding(
          padding: EdgeInsets.fromLTRB(20, 20, 20, 5),
          child: ResultExpansionList(),
        ),
      ],
    );
  }

  Widget _conjugationMetaDataCard(BuildContext context) {
    return Consumer<VerbTestModel>(builder: (context, testModel, child) {
      return ConjugationMetaDataCard(
          infinitive: testModel.infinitive,
          translation: testModel.translation,
          mode: testModel.mode,
          tense: testModel.tense);
    });
  }

  IconButton _settingsIcon(BuildContext context) {
    return IconButton(
        key: const Key('settingsButton'),
        icon: const Icon(Icons.settings),
        onPressed: () {
          Navigator.restorablePushNamed(context, SettingsView.routeName);
        });
  }

  IconButton _homeIcon(BuildContext context) {
    final VerbTestModel testModel =
        Provider.of<VerbTestModel>(context, listen: false);
    return IconButton(
        key: const Key('menu_key'),
        onPressed: () {
          testModel.reset();
          Navigator.restorablePopAndPushNamed(
              context, VerbSelectionRoute.routeName);
        },
        icon: const Icon(Icons.home));
  }

  IconButton _repeatIcon(BuildContext context) {
    final VerbTestModel testModel =
        Provider.of<VerbTestModel>(context, listen: false);
    return IconButton(
        key: const Key('repeat_key'),
        onPressed: () {
          testModel.reset();
          Navigator.popAndPushNamed(
              context, VerbStudymodeSelectionRoute.routeName);
        },
        icon: const Icon(Icons.refresh));
  }

  Wrap _titleCell(BuildContext context) {
    return Wrap(direction: Axis.horizontal, children: const [
      Text(
        "Rapport",
        textAlign: TextAlign.center,
      ),
    ]);
  }
}

class ResultExpansionList extends StatefulWidget {
  const ResultExpansionList({Key? key}) : super(key: key);

  @override
  State<StatefulWidget> createState() => ResultExpansionListState();
}

class ResultExpansionListState extends State<ResultExpansionList> {
  bool isExpanded = false;

  @override
  Widget build(BuildContext context) {
    return ExpansionPanelList(
      expansionCallback: (int index, bool state) {
        setState(() {
          isExpanded = !isExpanded;
        });
      },
      children: [
        ExpansionPanel(
          canTapOnHeader: true,
          isExpanded: isExpanded,
          headerBuilder: (BuildContext context, bool isExpanded) {
            return _scoreWidget(context);
          },
          body: SizedBox(
            height: 400,
            child: ListView(
                key: UniqueKey(),
                scrollDirection: Axis.vertical,
                children: _bodyListTiles(context)),
          ),
        ),
      ],
    );
  }

  Widget _scoreWidget(BuildContext context) {
    VerbTestModel model = context.watch<VerbTestModel>();
    return ScoreWidget(model.score, model.items.length);
  }

  List<Widget> _bodyListTiles(BuildContext context) {
    List<Widget> widgets = [];

    List<Widget> rows = [];
    for (int i = 0; i < _itemCount(context); i++) {
      rows.add(Card(
        key: UniqueKey(),
        elevation: 10,
        child: Padding(
          key: UniqueKey(),
          padding: const EdgeInsets.all(5.0),
          child: _resultCell(context, i),
        ),
      ));
    }

    widgets.add(Padding(
      key: UniqueKey(),
      padding: const EdgeInsets.fromLTRB(20, 0, 20, 0),
      child: Card(
        key: UniqueKey(),
        child: Column(
          key: UniqueKey(),
          children: rows,
        ),
      ),
    ));

    return widgets;
  }

  int _itemCount(BuildContext context) {
    return Provider.of<VerbTestModel>(context, listen: false).items.length;
  }

  Widget _resultCell(BuildContext context, person) {
    VerbTestModel model = context.watch<VerbTestModel>();
    String actual = model.items[person];
    String expected = model.expected[person];
    return EvaluationRow(person, model.targetLanguageIso,
        model.pronouns[person], actual, expected);
  }
}
