import 'dart:convert';

import 'package:cramr_ui_lib/model/practice_session.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:cramr_ui_lib/model/streak_engine.dart';

void main() {
  group('General tests for streak engine', () {
    test('simple test of StreakEngine', () {
      StreakEngine engine = StreakEngine();
      expect(engine.hasSessions(), false);
    });
    test('test number of sessions and add method', () {
      StreakEngine engine = StreakEngine();
      expect(engine.numberOfSessions(), 0);
      engine.addSession(PracticeSession(
          DateTime.now(), '_mode', '_tense', [], StudyMethod.flashcardMethod));
      expect(engine.numberOfSessions(), 1);
      engine.addSession(PracticeSession(
          DateTime.now(), '_mode', '_tense', [], StudyMethod.flashcardMethod));
      expect(engine.numberOfSessions(), 2);
    });
    test('test for presence of session on current day', () {
      StreakEngine engine = StreakEngine();
      var today = DateTime.now();
      engine.addSession(PracticeSession(
          today, '_mode', '_tense', [], StudyMethod.flashcardMethod));
      PracticeSession lastSession = engine.lastSession();
      var dateKey = lastSession.dateKey();
      expect(dateKey.year, today.year);
      expect(dateKey.month, today.month);
      expect(dateKey.day, today.day);
    });
    test('test for three day streak until today', () {
      var oneDay = const Duration(days: 1);
      var today = DateTime.now();
      var yesterday = today.subtract(oneDay);
      var dayBeforeYesterday = yesterday.subtract(oneDay);
      StreakEngine engine = StreakEngine();
      engine.addSession(PracticeSession(dayBeforeYesterday, '_mode', '_tense',
          [], StudyMethod.flashcardMethod));
      engine.addSession(PracticeSession(
          yesterday, '_mode', '_tense', [], StudyMethod.flashcardMethod));
      engine.addSession(PracticeSession(
          today, '_mode', '_tense', [], StudyMethod.writtenMethod));
      expect(engine.hasValidStreak(), true);
      expect(engine.streakCount(), 3);
    });
    test('test for skipped day with hole of 1 day', () {
      var oneDay = const Duration(days: 1);
      var today = DateTime.now();
      var yesterday = today.subtract(oneDay);
      var dayBeforeYesterday = yesterday.subtract(oneDay);
      StreakEngine engine = StreakEngine();
      engine.addSession(PracticeSession(
          dayBeforeYesterday, '_mode', '_tense', [], StudyMethod.coverMethod));
      engine.addSession(PracticeSession(
          today, '_mode', '_tense', [], StudyMethod.flashcardMethod));
      expect(engine.hasValidStreak(), false);
      expect(engine.streakCount(), 1);
    });
    test('test for gap of several days', () {
      var oneDay = const Duration(days: 1);
      var today = DateTime.now();
      var yesterday = today.subtract(oneDay);
      var dayBeforeYesterday = yesterday.subtract(oneDay);
      var threeDaysAgo = dayBeforeYesterday.subtract(oneDay);
      var fourDaysAgo = threeDaysAgo.subtract(oneDay);
      StreakEngine engine = StreakEngine();
      engine.addSession(PracticeSession(
          fourDaysAgo, '_mode', '_tense', [], StudyMethod.flashcardMethod));
      engine.addSession(PracticeSession(
          today, '_mode', '_tense', [], StudyMethod.writtenMethod));
      expect(engine.hasValidStreak(), false);
      expect(engine.streakCount(), 1);
    });
    test('find sessions for given date', () {
      StreakEngine engine = StreakEngine();
      DateTime today = DateTime.now();
      engine.addSession(PracticeSession(
          today, '_mode', '_tense', [], StudyMethod.writtenMethod));
      engine.addSession(PracticeSession(
          today, '_mode', '_tense', [], StudyMethod.coverMethod));
      DateKey expectedDateKey = DateKey(today.year, today.month, today.day);
      expect(engine.sessionsForDateKey(expectedDateKey).length, 2);
    });
    test('test the JSON methods', () {
      StreakEngine engine = StreakEngine();
      DateTime today = DateTime.now();
      engine.addSession(PracticeSession(
          today, '_mode', '_tense', [], StudyMethod.writtenMethod));
      String encoded = jsonEncode(engine);
      var decodeMap = jsonDecode(encoded);
      StreakEngine parsed = StreakEngine.fromJson(decodeMap);
      expect(parsed.sessions.length, engine.sessions.length);
    });
  });
}
